package de.tudarmstadt.langtech.substituter;

import java.util.Arrays;
import java.util.HashMap;
import java.util.List;

/**
 * Substitution for a single word
 * Usage is struct like.
 * @author cwirth
 *
 */
public class Substitution {

	private final String Sense,target;
	private final float confidence;

	List<String[]> Substitutions;

	public Substitution(String target,String sense,float confidence, HashMap<String, List<String[]>> substitutionMap) {
		this.Sense = sense;
		this.target = target;
		if(!sense.contains("@@")) {
			String checkstring=sense+"@@";
			int i=1;
			while(i<10 && ((substitutionMap.get(checkstring+i)==null) || (substitutionMap.get(checkstring+i).equals("")))) {
				i++;
			}
			if(i<10) {
				Substitutions = substitutionMap.get(checkstring+i);
			}
		} else {
			Substitutions = substitutionMap.get(sense);
		}
		this.confidence = confidence;
	}

	/**
	 * @return the sense class
	 */
	public String getSense() {
		return Sense;
	}

	/**
	 * @return the target (can be in another form than the fullform word)
	 */
	public String getTarget() {
		return target;
	}

	/**
	 * @return the confidence
	 */
	public float getConfidence() {
		return confidence;
	}

	/**
	 * @return list of possible substitutions with occurance counts
	 */
	public List<String[]> getSubstitutions() {
		return Substitutions;
	}

	@Override
	public String toString() {
		String ret = "Sense:"+Sense+" Target:"+target+" Confidence:"+confidence+":";
		if(Substitutions!=null) {
			for(String[] sub : Substitutions) {
				ret+=Arrays.toString(sub);
			}
		}
		return ret;
	}

}
