package de.tudarmstadt.langtech.substituter.run;

import java.io.StringReader;
import java.util.Arrays;
import java.util.logging.Level;
import java.util.logging.Logger;

import twsi.edu.stanford.nlp.process.PTBLexer;
import twsi.edu.stanford.nlp.process.PTBTokenizer;
import twsi.edu.stanford.nlp.process.Tokenizer;
import twsi.edu.stanford.nlp.process.WordTokenFactory;

import de.tudarmstadt.langtech.substituter.MLSenseSubstituter;
import de.tudarmstadt.langtech.substituter.SenseSubstituter;
import de.tudarmstadt.langtech.substituter.Substitution;
import de.tudarmstadt.langtech.substituter.settings.Configuration;
import de.tudarmstadt.langtech.substituter.training.TrainModels;
import de.tudarmstadt.langtech.substituter.util.TextFile;
import de.tudarmstadt.langtech.substituter.util.Util;
import de.tudarmstadt.langtech.viterbitagger.Viterbitagger;

/**
 * Main class. See README.txt or call main(args) for arguments
 * @author cwirth
 *
 */
public class Main {

	public static void main(String[] args) {
		// logger: deactivate debug information for PTB Lexer
		Logger LOGGER = Logger.getLogger(PTBLexer.class.getName());
		LOGGER.setLevel(Level.OFF);

		if(args.length != 0 && args[0].toLowerCase().equals("train")) {
			if(args.length <= 1) {
				System.out.println("Argument(s) expected: [train] [-c|-configfile] CONFIGFILE [-t|-trainfile] [4COLTRAININGDATA]");
			} else {
				int c;
				boolean found=false;
				if((c=Arrays.asList(args).indexOf("-c")) != -1 || (c=Arrays.asList(args).indexOf("--configfile")) != -1) {
					Configuration.setConfigFile(args[c+1],true);
					found=true;
				} else {
					Configuration.setConfigFile(args[1],true);
				}
				if((c=Arrays.asList(args).indexOf("-t")) != -1 || (c=Arrays.asList(args).indexOf("--trainfile")) != -1) {
					Configuration.trainingsData=args[c+1];
				} else {
					if(args.length>=4 || (args.length>=3 && !found)) {
						Configuration.trainingsData=args[args.length-1];
					}
				}
				// Train Call
				TrainModels.train();
			}

			// run execution block: processing pipeline
		} else if(args.length != 0 && args[0].toLowerCase().equals("run")) {
			if(args.length < 3) {
				System.out.println("Argument(s) expected: [run] [-c|-configfile] CONFIGFILE [-t|-textfile] TEXTtoSUBSTITUE");
			} else {
				int c;
				SenseSubstituter ss;
				if((c=Arrays.asList(args).indexOf("-c")) != -1 || (c=Arrays.asList(args).indexOf("-configfile")) != -1) {
					ss = new MLSenseSubstituter(args[c+1]);
				} else {
					ss = new MLSenseSubstituter(args[1]);
				}
				TextFile file;
				if((c=Arrays.asList(args).indexOf("-t")) != -1 || (c=Arrays.asList(args).indexOf("-textfile")) != -1) {
					file = new TextFile(args[c+1]);
				} else {
					file = new TextFile(args[args.length-1]);
				}

				Viterbitagger tagger=new Viterbitagger(Configuration.taggerModelPrefix);
				for(String line : file) {
					line = line.replace("<b>","").replace("</b>", ""); // legacy for TWSI format
					String sentence = Util.tokenize(line);
					String taggedSentence="%^%/BOS"+tagger.addPOStags(sentence)+" %$%/EOS";
					sentence="%^% "+sentence+" %$%";

					// prepare words and POS tags
					String[] words = sentence.replace("  ", " ").split(" ");
					String[] wordTag= taggedSentence.replace("  ", " ").split(" ");
					String[] postags=new String[words.length];
					for(int i=0;i<wordTag.length;i++) {
						String parts[]=wordTag[i].split("/");
						postags[i]=parts[1];
					}
					for(int i=1;i<words.length-1;i++) {
						Substitution result = ss.getSubstitution(i, words, postags);
						if(result!=null) {
							// write out substitutions: change here for different format
							System.out.print("<target=\""+words[i]+"\" lemma=\""+result.getTarget()+"\" sense=\""+result.getSense()+"\" confidence=\""+result.getConfidence()+"\" substitutions=\"");
							if(result.getSubstitutions()!=null) {
								for(String[] subs : result.getSubstitutions()) {
									System.out.print("["+subs[0]+","+subs[1]+"]");
								}
							}
							System.out.print("\"> ");
						} else {
							System.out.print(words[i]+" ");
						}
					}
					System.out.print("\n");
				}
			}
		} else {
			System.out.println("Argument(s) expected: [run|train] [-c|--configfile] CONFIGFILE [[-t|--textfile] TEXTtoSUBSTITUE | [-tr|--trainfile] [4COLTRAININGDATA]]");
		}

	}

}
