package de.tudarmstadt.langtech.substituter.settings;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.UnsupportedEncodingException;
import java.util.Properties;

/**
 * Handles all configuration parameters, holds configuration data from config file
 *
 * see README.txt for details
 *
 * @author cwirth
 *
 */
public class Configuration {

	public static String trainingsData="allSentences.txt";

	public static String ambiguousWordsFile="conf/TWSI1_ambiguous_words.txt";

	public static String inventoryFile="conf/inventories.txt";
	public static String substitutionsFile="conf/TWSI2-substitutions.txt";
	public static String monosemousWordsFile="conf/targets-singlesense.txt";
	public static String lemmaMapFile="conf/lemma_fullform.txt";

	public static String modelFolder="models";
	public static String taggerModelPrefix="posdata/ptb3-tagger";

	public static String wekaClassifier="weka.classifiers.functions.SMO";
	public static String wekaOptions="";

	public static boolean StanfordTokenizer=true;

    public static String home = "";
    public static String taggerModelHome = "";

	/**
	 * Set config per config file
	 * @param filename the file, see README.txt
	 */
	public static void setConfigFile(String filename,boolean isTrain) {
		Properties p = new Properties();
		if(!new File(filename).exists()) {
			System.out.println("[Error] configuration not found.");
			System.exit(0);
		}
		try {
			p.load(new BufferedReader(new InputStreamReader(new FileInputStream(filename), "UTF-8")));
		} catch (UnsupportedEncodingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		setConfigFile(p);
		if(!validate(isTrain)) {
			System.exit(0);
		}
	}

	/**
	 * Sets config via a properties list
	 * @param p
	 */
	public static void setConfigFile(Properties p) {
		Configuration.home = p.getProperty("mainDir",Configuration.home);
		if(! new File(Configuration.home).exists()) {
			System.out.println("[Warning] mainDir not found, setting to default !");
			Configuration.home = "";
		}
        Configuration.taggerModelHome = p.getProperty("taggerModelDir",Configuration.taggerModelHome);
        if(! new File(Configuration.taggerModelHome).exists()) {
            System.out.println("[Warning] taggerModelDir not found, setting to default !");
            Configuration.taggerModelHome = Configuration.home;
        }
		Configuration.trainingsData=home+p.getProperty("trainingsData",Configuration.trainingsData);
		Configuration.ambiguousWordsFile=home+p.getProperty("ambiguousWordsFile",Configuration.ambiguousWordsFile);
		Configuration.inventoryFile=home+p.getProperty("inventoryFile",Configuration.inventoryFile);
		Configuration.substitutionsFile=home+p.getProperty("substitutionsFile",Configuration.substitutionsFile);
		Configuration.monosemousWordsFile=home+p.getProperty("monosemousWordsFile",Configuration.monosemousWordsFile);
		Configuration.lemmaMapFile=home+p.getProperty("lemmaMapFile",Configuration.lemmaMapFile);
		Configuration.modelFolder=home+p.getProperty("modelFolder",Configuration.modelFolder);
		Configuration.taggerModelPrefix=taggerModelHome+p.getProperty("taggerModelPrefix",Configuration.taggerModelPrefix);
		Configuration.wekaClassifier=p.getProperty("wekaClassifier",Configuration.wekaClassifier);
		Configuration.wekaOptions=p.getProperty("wekaOptions",Configuration.wekaOptions);
		Configuration.StanfordTokenizer=p.getProperty("tokenization","StanfordTokenizer").toLowerCase().equals("whitespace")?false:true;
	}

	private static boolean validate(boolean isTrain) {
		boolean result=true;
		File test = new File(Configuration.trainingsData);
		if(!test.exists() && isTrain) {
			System.out.println("[Error] trainingsData file not found");
			result=false;
		}
		test = new File(Configuration.ambiguousWordsFile);
		if(!test.exists()) {
			System.out.println("[Error] ambiguousWordsFile file not found");
			result=false;
		}
		test = new File(Configuration.inventoryFile);
		if(!test.exists()) {
			System.out.println("[Error] inventoryFile file not found");
			result=false;
		}
		test = new File(Configuration.substitutionsFile);
		if(!test.exists()) {
			System.out.println("[Error] substitutionsFile file not found");
			result=false;
		}
		test = new File(Configuration.monosemousWordsFile);
		if(!test.exists()) {
			System.out.println("[Error] monosemousWordsFile file not found");
			result=false;
		}
		test = new File(Configuration.lemmaMapFile);
		if(!test.exists()) {
			System.out.println("[Error] lemmaMapFile file not found");
			result=false;
		}
		test = new File(Configuration.modelFolder);
		if((!test.exists() || !test.isDirectory() || test.list().length==0) && !isTrain) {
			System.out.println("[Error] modelFolder folder not existent or empty");
			result=false;
		}
		test = new File(Configuration.taggerModelPrefix+".model");
		if(!test.exists()) {
			System.out.println("[Error] taggerModels not found");
			result=false;
		}
		try {
			Class.forName(Configuration.wekaClassifier);
		} catch (ClassNotFoundException e) {
			System.out.println("[Error] classifier not found");
			result=false;
		}
		/*try {
			if(weka.core.Utils.splitOptions(Configuration.wekaOptions).length == 0) {
				System.out.println("[Warning] weka options are empty or not correctly formatted");
			}
		} catch (Exception e) {
			System.out.println("[Warning] weka options are empty or not correctly formatted");
		}*/
		return result;
	}

}
