/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package de.tudarmstadt.langtech.substituter.training;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.StringReader;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

import twsi.edu.stanford.nlp.process.PTBTokenizer;
import twsi.edu.stanford.nlp.process.Tokenizer;
import twsi.edu.stanford.nlp.process.WordTokenFactory;

import de.tudarmstadt.langtech.substituter.settings.Configuration;
import de.tudarmstadt.langtech.substituter.util.Inventory;
import de.tudarmstadt.langtech.substituter.util.TextFile;
import de.tudarmstadt.langtech.substituter.util.Util;
import de.tudarmstadt.langtech.viterbitagger.Viterbitagger;
/**
 * Handles the extraction of features from the training data and writes ARFF files for use in WEKa
 * @author biem, cwirth
 */
public class ModelPreparer {




	boolean verbose=false;

	Viterbitagger tagger;
	Inventory[] inventories;


	/** init function, gets jhhanfles on components like tagger)
	 * @param targs */
	public void init(Viterbitagger t, String[] inventoryfiles, Vector targs) {
		this.tagger=t;
		// load inventories
		inventories=new Inventory[inventoryfiles.length];

		for(int i=0;i<inventoryfiles.length;i++) {
			inventories[i]=new Inventory(Util.inventoryEntrySuffix,Util.usePos);
			inventories[i].init(inventoryfiles[i],targs);
		} // rof


	}

	/**
	 * Creates an ARFF file (and saves it) based upon the trainingsdate in the instancesfile
	 * @param target
	 * @param instancesfilename
	 * @param outfilename
	 */
	public void writeARFFfile(String target, String instancesfilename, String outfilename) {
		System.out.println("Target: "+target);
		// init storage for feature values
		Hashtable[] featVals=null;
		// go through instancesfile and compute features
		TextFile instancesfile=new TextFile(instancesfilename);
		// saves data
		String databuffer=new String();
		boolean found=false;
		for(String line : instancesfile) {
			String[] lcols=line.split("\t");
			Vector feats=new Vector();
			if (lcols.length>3) {
				// this is 4-col tab-separated : class, target(lemma), match(fullform in sentence), sentence
				String classification=lcols[0];
				String thisTarg=lcols[1];
				String thisMatch=lcols[2];

				// only proceed if line contains our current target -- other lines in the training file are ignored
				if (thisTarg.equals(target)) {
					found=true;
					String sentence = "";
					String line2 =lcols[3].replace("<b>","").replace("</b>", ""); // legacy for TWSI data
					sentence = Util.tokenize(line2);
					String taggedSentence="%^%/BOS"+tagger.addPOStags(sentence)+" %$%/EOS";
					//System.out.println(taggedSentence);
					sentence="%^% "+sentence+" %$%";  // no markup, adding BOS/EOS
					int matchpos=-1; // matching position in word stream

					// prepare words and POS tags
					String[] words = sentence.replace("  ", " ").split(" ");
					words = Util.makeWekaCompatible(words);
					String[] wordTag= taggedSentence.replace("  ", " ").split(" ");
					String[] postags=new String[words.length];
					for(int i=0;i<wordTag.length;i++) {
						String parts[]=wordTag[i].split("/");
						postags[i]=parts[1];
						//System.out.println("Word: "+words[i]);
					}

					// get (last) matchpos
					matchpos=-1;
					for(int i=0;i<words.length;i++) {
						if (words[i].equals(thisMatch)) {matchpos=i;}
					} // rof get matchpos

					feats=Util.sentenceToFeatures(words, postags, matchpos,inventories);  // build features
					// target class
					feats.add(classification);
					// write feats of this instance to line representation
					String outline="";


					if(featVals==null) featVals=new Hashtable[feats.size()];
					
					for (int i=0;i<feats.size();i++) {
						String nextFeature=(String)feats.elementAt(i);

						//System.out.println(i+":"+nextFeature);

						if ((nextFeature==null)||(nextFeature.equals(""))) {nextFeature="?";} else {
							// add to values
							if (featVals[i]==null) {featVals[i]=new Hashtable();}
							featVals[i].put(nextFeature,"true");
						}
						if (i>0) {outline+=","+nextFeature;} else {outline+=nextFeature;}
					}

					if (feats.size()>0) {
						databuffer+=outline+"\n";
					}
					if (verbose) {
						System.out.println(outline);
					}
				}
			}
		} // rof file

		// write file if any training instance of the target was found
		if(found) {
			try {
				File file = new File(outfilename);
				System.out.println(file.getAbsolutePath());
				BufferedWriter out = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(outfilename), "UTF-8"));
				// header
				out.write("@relation "+target+"_wsd\n");
				for(int i=0;i<featVals.length;i++) {
					out.write("@attribute attribute"+i+" {");
					boolean first=true;
					//if (verbose) System.out.println("Writing feature nr: "+i);
					if (featVals[i]!=null) {
						for(Enumeration e=featVals[i].keys();e.hasMoreElements();) {
							if (!first) {out.write(", ");}

							out.write((String)e.nextElement());

							first=false;
						} // rof keys i
					} else {
						out.write("?");
					}
					out.write("}\n");
				} // rof i

				// data
				out.write("@data\n");
				out.write(databuffer);
				out.close();
			} catch (IOException e) {
				System.out.println("IOException: ");
				e.printStackTrace();
			} // end catch
		} else {
			System.out.println("[Error] No traingsdata for target in file, but target was declared ambiguous.");
		}
	} // end writeARFFfile

}
