/*
 * contains inventory of triggers: words mapping to cluster class
 */
package de.tudarmstadt.langtech.substituter.util;
import java.io.File;
import java.lang.reflect.Array;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Enumeration;
import java.util.List;
import java.util.TreeMap;
import java.util.Vector;

import de.tudarmstadt.langtech.substituter.settings.Configuration;



/**
 * Represents an inventory, defining a word->clusterID mapping.
 * @author biem, cwirth
 */
public class Inventory {


	boolean d=!true; // debug flag
	TreeMap<String,String> targetToMap;
	private String required;
	private boolean usePos = true;
	HashMap<String,Integer> classToID;

	public Inventory() {
		required = null;
	}
	/**
	 * @param requiredString all targets must end with this string (limits to /NN targets e.g.)
	 * @param usePos if false, the pos tag of words is ignored 
	 */
	public Inventory(String requiredString, boolean usePos) {
		required = requiredString;
		this.usePos = usePos;
	}
	/** initializes inventory by reading it from plaintext file 
	 * @param targs */    
	public Inventory init(String filename, Vector targs) {
		return init_unweighted(Configuration.home+filename,targs);
	}

	public Inventory init_unweighted(String filename, Vector targs) {
		if(!new File(filename).exists()) {
			System.out.println("[Error] Inventoryfile: "+filename+" not found !");
			return null;
		}
		targetToMap = new TreeMap<String,String>();
		classToID = new HashMap<String,Integer>();
		TextFile inputfile=new TextFile(filename);
		//System.out.println("This file: "+filename);
		int lineNr =0;
		for(String line : inputfile) {
			String lcols[]=line.split("\t");
			String target=lcols[0];
			if(required == null || target.contains(required)) {
				if(required != null || (!usePos && target.indexOf("/") != -1))
					target = target.substring(0,target.indexOf("/"));
				if(targs.contains(target)) {
					if(classToID.containsKey(target)) {
						target = Integer.toString(classToID.get(target));
					} else {
						classToID.put(target,classToID.size());
					}
					//System.out.println("Target: "+target);
					String tclass=target+"@"+lcols[1];
					String[] features=lcols[2].split(", ");

					for (int i=0;i<features.length;i++) {
						if(!usePos && features[i].indexOf("/") != -1) {
							features[i] = features[i].substring(0,features[i].indexOf("/"));
						}
						targetToMap.put(target+features[i],tclass);
						//if (d) System.out.println("Inserting "+tclass+" for "+features[i]+", target="+target);
					} // rof i features

					//if (d) System.out.println("created inventory for "+targetToMap.size()+" targets");
				}
				if(d && lineNr%1000==0) System.out.println(lineNr); 
				lineNr++;
			}
		} // rof line
		return this;
	} // end init


	public String getClusterId(String target, String feature, String postag) {
		String retClass="";
		String key = classToID.get(target)+feature;
		if(postag!=null) key+="/"+postag;
		if (targetToMap.containsKey(key)) {
			if (d) System.out.println("Found target: "+target);
			retClass=(String)targetToMap.get(key);
		} // foi target ddefined
		else {
			if (d) System.out.println(" target not found: "+target);
		}

		return retClass;
	} // end getClusterId

}
