package de.tudarmstadt.langtech.substituter.util;

import java.io.File;
import java.io.FileInputStream;
import java.io.ObjectInputStream;
import java.io.StringReader;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.List;
import java.util.Vector;

import de.tudarmstadt.langtech.substituter.settings.Configuration;

import twsi.edu.stanford.nlp.process.PTBTokenizer;
import twsi.edu.stanford.nlp.process.Tokenizer;
import twsi.edu.stanford.nlp.process.WordTokenFactory;
import weka.classifiers.Classifier;
import weka.core.Instance;
import weka.core.Instances;

public class Util {

	//Feature switches: must be consistent with the model creation parameters
	static boolean s_onewordwindow=true;   //(2)  words -1,+1
	static boolean s_pos_2words=true;      //(2)    pos sequences -2-1 and +1+2
	static boolean s_pos_1word=true;       //(3)    pos tags -1,0,+1
	static boolean s_nouncontext=true;     //(4)    NN* tag words -2-1+1+2
	static boolean s_verbcontext=true;     //(2)    V* tag words -1+1
	static boolean s_adjcontext=true;      //(2)    JJ* tag words -1+1
	static boolean s_ppcontext=true;       //(2)    word with tags IN, RB, TO , RP  -1+1
	static boolean s_inventories=true;     //(#InventoryFiles) |inventories|   inventory assignments

	public static String inventoryEntrySuffix = "/NN"; //in inventories, only read in noun entries (in case of verb entries)
	public static boolean usePos = true;  // for matching inventory entries with sentence

	/** Cluster Feature assignment function
	 * counts number of features per class for a sequence of words and returns highest class or ""
	 * @param postags */

	public static String assignClusterFeatureToContext(Inventory inv, String target, String[] words, String[] postags) {

		Hashtable featureCounts=new Hashtable();
		// take off dot part from target
		String tparts[]=target.split("\\.");
		target=tparts[0];

		for(int i=0;i<words.length;i++) {
			String thisId;
			if(postags!=null) {
				thisId=inv.getClusterId(target, words[i], postags[i]);
			}
			else {
				thisId=inv.getClusterId(target, words[i], null);
			}
			if (!(thisId.equals(""))) {
				if (featureCounts.containsKey(thisId)) {
					featureCounts.put(thisId, new Integer(1+((Integer)featureCounts.get(thisId))).intValue());
				} else {
					featureCounts.put(thisId, new Integer(1));
				}
			} // fi ""
		} // rof i

		String maxClass="";
		int maxCount=0;
		for(Enumeration e=featureCounts.keys();e.hasMoreElements();) {
			String currClass=(String)e.nextElement();
			int thisCount=((Integer)featureCounts.get(currClass)).intValue();
			if (thisCount>maxCount) {
				maxClass=currClass;
				maxCount=thisCount;
			}

		}

		return maxClass;
	}
	/**
	 * Creates a feature vector out of the given information
	 * @param words
	 * @param postags
	 * @param matchpos
	 * @param inventories
	 * @return
	 */
	public static Vector sentenceToFeatures(String[] words,String[] postags, int matchpos,Inventory[] inventories) {

		Vector feats=new Vector();

		int currpos; // var used for current pos in various places



		if (matchpos==-1) {
			System.out.println("WARNING: no match found in instance");
			//for(int i=0;i<words.length;i++) {
			//   if (words[i].equals(thisMatch)) {System.out.println("MATCH!");}
			//   System.out.println(words[i]);
			//} // rof get matchpos
		} else {

			// immediate context features
			if (s_onewordwindow) {
				feats.add(words[matchpos-1]);
				feats.add(words[matchpos+1]);
			}

			// pos
			if (s_pos_1word) {
				feats.add(postags[matchpos-1]);
				feats.add(postags[matchpos]);
				feats.add(postags[matchpos+1]);
			}
			if (s_pos_2words) {
				if (matchpos>=2) {
					feats.add(postags[matchpos-2]+"_"+postags[matchpos-1]);
				} else {
					feats.add("");
				}
				if (matchpos+2<postags.length) {
					feats.add(postags[matchpos+1]+"_"+postags[matchpos+2]);
				} else {
					feats.add("");
				}
			}

			// content word windows

			// nouns +-2
			if (s_nouncontext) {
				String[] nounsAround= new String[4];
				currpos=1;
				for(int i=matchpos-1;i>=0;i--) {
					if (currpos>=0) {
						if ((postags[i]!=null)&&(postags[i].equals("NN")||postags[i].equals("NNS"))) {
							nounsAround[currpos]=words[i];
							currpos--;
						}
					}
				} // rof i--
				//System.out.println(target+" "+taggedSentence);

				currpos=2;
				for(int i=matchpos+1;i<postags.length;i++) {
					//System.out.println(target+" "+words[i]+" "+postags[i]);
					if (currpos<=3) {
						if ((postags[i]!=null)&&(postags[i].equals("NN")||postags[i].equals("NNS"))) {
							nounsAround[currpos]=words[i];
							currpos++;
						}
					}
				} // rof i++

				feats.add(nounsAround[0]);
				feats.add(nounsAround[1]);
				feats.add(nounsAround[2]);
				feats.add(nounsAround[3]);
			}


			// verbs +-1
			if (s_verbcontext) {
				String[] verbAround= new String[2];
				currpos=0;
				for(int i=matchpos-1;i>=0;i--) {
					if (currpos>=0) {
						if ((postags[i]!=null)&&(postags[i].startsWith("V"))) {
							verbAround[currpos]=words[i];
							currpos--;
						}
					}
				} // rof i--

				currpos=1;
				for(int i=matchpos+1;i<postags.length;i++) {
					//System.out.println(target+" "+words[i]+" "+postags[i]);
					if (currpos<=1) {
						if ((postags[i]!=null)&&(postags[i].startsWith("V"))) {
							verbAround[currpos]=words[i];
							currpos++;
						}
					}
				} // rof i++
				feats.add(verbAround[0]);
				feats.add(verbAround[1]);
			}

			// adj +-1
			if (s_adjcontext) {
				String[] adjAround= new String[2];
				currpos=0;
				for(int i=matchpos-1;i>=0;i--) {
					if (currpos>=0) {
						if ((postags[i]!=null)&&(postags[i].startsWith("JJ"))) {
							adjAround[currpos]=words[i];
							currpos--;
						}
					}
				} // rof i--

				currpos=1;
				for(int i=matchpos+1;i<postags.length;i++) {
					//System.out.println(target+" "+words[i]+" "+postags[i]);
					if (currpos<=1) {
						if ((postags[i]!=null)&&(postags[i].startsWith("JJ"))) {
							adjAround[currpos]=words[i];
							currpos++;
						}
					}
				} // rof i++
				feats.add(adjAround[0]);
				feats.add(adjAround[1]);
			}

			//IN, RB, TO , RP  +-1
			if (s_ppcontext) {
				String[] adjAround= new String[2];
				currpos=0;
				for(int i=matchpos-1;i>=0;i--) {
					if (currpos>=0) {
						if ((postags[i]!=null)&&((postags[i].startsWith("IN"))||postags[i].startsWith("RB")||postags[i].startsWith("TO")||postags[i].startsWith("RP"))) {
							adjAround[currpos]=words[i];
							currpos--;
						}
					}
				} // rof i--

				currpos=1;
				for(int i=matchpos+1;i<postags.length;i++) {
					//System.out.println(target+" "+words[i]+" "+postags[i]);
					if (currpos<=1) {
						if ((postags[i]!=null)&&((postags[i].startsWith("IN"))||postags[i].startsWith("RB")||postags[i].startsWith("TO")||postags[i].startsWith("RP"))) {
							adjAround[currpos]=words[i];
							currpos++;
						}
					}
				} // rof i++
				feats.add(adjAround[0]);
				feats.add(adjAround[1]);
			}

			// inventories
			if (s_inventories) {
				for (Inventory inventorie : inventories) {
					String disamb;
					if(inventoryEntrySuffix!=null) {
						disamb=assignClusterFeatureToContext(inventorie,words[matchpos],words,postags);
					}
					else {
						disamb=assignClusterFeatureToContext(inventorie,words[matchpos],words,null);
					}
					feats.add(disamb);

				} // rof inv
			}

		} // esle matchpos -1
		return feats;
	}

	/**
	 * Creates a Weka Instance object
	 * @param values feature vector
	 * @param dataset example weka dataset for feature counts/types
	 * @return
	 */
	public static Instance makeCompatible(Vector<String> values, Instances dataset) {
		Instance retInstance=new Instance(values.size()+1);
		// dataset knows about possible nominal values
		retInstance.setDataset(dataset);
		for (int i = 0; i < values.size(); i++) {
			// check for existence of nominal values, otherwise convert to "missing": unknown vocabulary as features
			if (values.get(i)==null) {retInstance.setMissing(i);}
			else if (dataset.attribute(i).indexOfValue(values.get(i))>=0) {//  ((possValues[i]!=null)&&(possValues[i].containsKey(values[i]))) {
				retInstance.setValue(i, values.get(i));

			} else {retInstance.setMissing(i);}
		}
		return retInstance;
	}

	/**
	 * reads all inventories
	 * @param inventoryListFile a filename, see README.txt
	 * @return
	 */
	public static String[] initInventories(String inventoryListFile) {
		TextFile instancesfile=new TextFile(inventoryListFile);
		int size=0;
		int lineNr=0;
		String[] inventories=new String[0];
		for(String line : instancesfile) {
			if(line != null && line.length() > 0) {
				if (lineNr==0) {
					size=(new Integer(line)).intValue(); // firtst line: size to be considered
					inventories=new String[size];
				} else {
					inventories[lineNr-1]=line;
				}

				lineNr+=1;
			}
		}
		return inventories;
	} // end initInventories

	/** reads the ambigious words list */
	public static Vector readTargets(String filename) {
		Vector retvec=new Vector();
		TextFile instancesfile=new TextFile(filename);

		// reads in 1st col
		for(String line : instancesfile) {
			String[] cols=line.split("\t");
			retvec.add(cols[0]);

		} // rof line
		return retvec;
	}

	/** loads classifier from model file
	 *
	 * @param filename
	 * @return classifier
	 */
	public static Classifier loadClassifier(File filename) {
		Classifier cModel;
		try {
			ObjectInputStream ois = new ObjectInputStream(new FileInputStream(filename));
			cModel = (Classifier) ois.readObject();
			return cModel;
		} catch (Exception e) {System.err.println(e.getMessage());}
		return null;
	}

	/** loads dataset from ds file
	 *
	 * @param filename
	 * @return
	 */
	public static Instances loadDataset(File filename) {
		Instances inst;
		try {
			FileInputStream dsInFile = new FileInputStream(filename);
			ObjectInputStream dsInObjectFile = new ObjectInputStream(dsInFile);
			inst=(Instances)dsInObjectFile.readObject();
			dsInObjectFile.close();
			return inst;

		} catch (Exception e) { System.out.println("Error with DS reading: "+e.getMessage());}
		return null;
	}

	/**
	 * Inits the substitutions map
	 * @param substitutionFile a filename, see README.txt
	 * @return
	 */
	public static HashMap<String,List<String[]>> initSubstitutions(String substitutionFile) {
		TextFile subfile=new TextFile(substitutionFile);
		HashMap<String,List<String[]>> returnMap = new HashMap<String,List<String[]>>();
		for(String line : subfile) {
			String[] data = line.split("\t");
			List<String[]> list;
			if(returnMap.containsKey(data[0])) {
				list = returnMap.get(data[0]);
			} else {
				list = new ArrayList<String[]>();
			}
			list.add(new String[]{data[2],data[3]});
			returnMap.put(data[0],list);
		}
		return returnMap;
	}
	
	/**
	 * Tokenizes the given line by Stanford or Withspace, depending on configuration.
	 * 
	 * @param line
	 * @return
	 */
	public static String tokenize(String line) {
		String sentence = "";
		if(Configuration.StanfordTokenizer) {
			// tokenizer parameters should be the same as in training
			Tokenizer tok = PTBTokenizer.factory(new WordTokenFactory(), "normalizeParentheses=false,normalizeOtherBrackets=false,americanize=false").getTokenizer(new StringReader(line));
			for(Object el : tok.tokenize()) {
				sentence+=el.toString()+" ";
			}
			if(sentence.endsWith(" ")) {
				sentence=sentence.substring(0,sentence.length()-1);
			}
		} else {
			sentence = line;
		}
		return sentence;
	}
	
	public static String[] makeWekaCompatible(String[] words) {
		String[] words2 = new String[words.length];
		// Weka compatibility replacements
		for(int i=0;i<words.length;i++) {
			words2[i]=words[i].replace(",","KOMMA");
			words2[i]=words2[i].replace("-","DASH");
			words2[i]=words2[i].replace(".","FS");
			words2[i]=words2[i].replace("?","QMARK");
			words2[i]=words2[i].replace("!","EMARK");
			words2[i]=words2[i].replace("\\","BSLASH");
			words2[i]=words2[i].replace("\"","DQUOTE");
			words2[i]=words2[i].replace("%","PERC");
			words2[i]=words2[i].replace("'","SQUOTE");
			words2[i]=words2[i].replace("{","OPENCURL");
			words2[i]=words2[i].replace("}","CLOSECURL");
		}
		return words2;
	}
	
	
}
