/*
 * Lexicon.java
 *
 * Created on 1. Dezember 2005, 20:34
 *
 * Lexicon stores the lexicon in a matrix
 * word1 p(t1) p(t2) 
 * word2 p(t1) p(t2) ..
 * ...
 *
 * as given in input file. 
*/
package de.tudarmstadt.langtech.viterbitagger;
import java.util.Hashtable;
import java.util.Vector;
import java.util.Enumeration;
import java.io.*;

import de.uni_leipzig.asv.utils.*;


/**
 *
 * @author BIEMANN
 */
public class Lexicon implements Serializable{
    
    boolean d=false; // switch for verbose output for debugging
    double[][] lexmatrix; // 2-dimensional matrix for word-tag probabilities
    Hashtable wordToRow;  // word list

    int rows; // how many rows
    int columns; // how many columns
    
    int sumFrequency;
    double SMOOTHPRETREEVAL=0.0001;
       
    //  pretrees
    Pretree guessTagFront;
    Pretree guessTagRev; 
    
    
    
    /** Creates a new instance of Lexicon */
    public Lexicon(String filename) {
        try {
           load(filename);
        } catch (Exception e) {// System.err.println("File IO problem with "+filename+" :"+e.getMessage()); 
        }        
    } // end constructor
    
    
    // Loads matrix from file
    private void load(String filename) throws IOException, FileNotFoundException {
        int linecount = 0;
        sumFrequency=0;
        String line;
        wordToRow=new Hashtable();   
        
        // Pretree on reversed word
        guessTagRev=new Pretree();
        guessTagRev.setReverse(true); // reverse
        guessTagRev.setIgnoreCase(false); // reverse

        // Pretree on word
        guessTagFront=new Pretree();
        guessTagFront.setReverse(false); // from front
        guessTagFront.setIgnoreCase(false); // reverse
        
        boolean pretreePrepared=false;
        // test pretree for existence
        File pretree_front_exists = new File(filename+".front.pretree");
        File pretree_rev_exists = new File(filename+".rev.pretree");
        if (pretree_front_exists.exists()&&pretree_rev_exists.exists()) {
            pretreePrepared=true;
            guessTagFront=new Pretree();
            guessTagFront.load(filename+".front.pretree");       
            guessTagRev=new Pretree();
            guessTagRev.load(filename+".rev.pretree");           
            if (d) System.out.println("Loaded existing pretrees for "+filename);
        }
        
            
        BufferedReader lexFileReader1 = new BufferedReader(new FileReader(filename));
	
        // First pass: Count lines for array dimensioning      
        line=lexFileReader1.readLine();
        String linearray[]=line.split("\t");
        columns=linearray.length-1; 
        
        while(lexFileReader1.readLine()!=null) {
		    linecount++;
        } // elihw
        lexFileReader1.close();
        rows=linecount+1;
        
        // Initialize matrix
        lexmatrix=new double[rows][columns];    
        
        // and fill it
        BufferedReader lexFileReader2 = new BufferedReader(new FileReader(filename));
        
        linecount=0;
        while((line=lexFileReader2.readLine())!=null) {
            linearray=line.split("\t");
            
            int freq=(new Integer(linearray[0])).intValue();
  
            sumFrequency+=freq;
            
            String newword=linearray[1];
            wordToRow.put(newword, new Integer(linecount));
                       
            for (int p=0;p<linearray.length-2;p++) {
               Double actPos= new Double(linearray[p+2]);
               double doubleActPos=actPos.doubleValue();
               lexmatrix[linecount][p]=doubleActPos;                   
               // insert in pretree    
                   
               int tagFreq=(int)(0.0001+((double)freq)*doubleActPos);
             
               if (tagFreq>0) {
                  if (false) System.out.println("[lex] inserting "+newword+" for tag "+p+" with freq "+tagFreq);           
                          
                  if (!pretreePrepared) {
                      guessTagFront.train(newword, ""+p, tagFreq);
                      guessTagRev.train(newword, ""+p, tagFreq);                      
                  } // fi pretreePrepared
               } // fi tagfreq    
             
            } // rof
              
            linecount++;
        } // elihw
        
        
        lexFileReader2.close();
        
          
        // Pretree toString format for memory saving: prune it, save it and load it. 
        if (!pretreePrepared) {
            guessTagFront.prune();
            guessTagFront.save(filename+".front.pretree");

            guessTagRev.prune();
            guessTagRev.save(filename+".rev.pretree");

            guessTagFront=new Pretree();
            guessTagFront.load(filename+".front.pretree");

            guessTagRev=new Pretree();
            guessTagRev.load(filename+".rev.pretree");
        }
    } // end private void load
    
    
    /** returns distribution over tags for a word by combining lexicon and pretree information */
    public double[] getTagDistribution(String word) {
        double[] retVal=new double[columns];
            
        // if in lexicon, return lexicon entry
        if (wordToRow.containsKey(word)) {
           for(int i=0;i<columns;i++) {
              retVal[i]=lexmatrix[((Integer)wordToRow.get(word)).intValue()][i];
           }
        } else { // otherwise, guess with pretree
                             
            Vector reasonF=guessTagFront.classDistribution(word);
            Hashtable rfhash=new Hashtable();
            int rfsum=0;
            for(Enumeration e=reasonF.elements();e.hasMoreElements();) {
                String elem=(String)e.nextElement();
                String a[]=elem.split("=");
                rfhash.put(a[0], a[1]);
                rfsum+=(new Integer(a[1])).intValue();
            } // rof enum e

            Vector reasonR=guessTagRev.classDistribution(word);
            Hashtable rrhash=new Hashtable();
            int rrsum=0;
            for(Enumeration e=reasonR.elements();e.hasMoreElements();) {
                String elem=(String)e.nextElement();
                String a[]=elem.split("=");
                rfhash.put(a[0], a[1]);
                rrsum+=(new Integer(a[1])).intValue();
            } // rof enum e

            // distribution computation
            for(int i=0;i<columns;i++) { 
                double frval=SMOOTHPRETREEVAL;
                double rrval=SMOOTHPRETREEVAL;
                if (rfhash.containsKey(i+"")) {frval+=(new Double((String)rfhash.get(i+""))).doubleValue()/((double)rfsum);}
                if (rrhash.containsKey(i+"")) {rrval+=(new Double((String)rrhash.get(i+""))).doubleValue()/((double)rrsum);}
                retVal[i]=frval*rrval;
            } // rof i
                
        } // esle guess w/ pretree
        
        return retVal;
    } // end public double tagDistribution    
    
 
    /** test whether word is in lexicon or not */
    public boolean containsWord(String word) {
        return wordToRow.containsKey(word);
    } // end public double containsWord(String word)
             
} // end public class Lexicon
