/*
 * TagList.java
 *
 * Maintains the list of tags. 
 * "-" Tag is invented for inventing tokens before the beginning of sentences.
 */

package de.tudarmstadt.langtech.viterbitagger;
import java.util.Hashtable;
import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.FileReader;
/**
 *
 * @author BIEMANN
 */
public class TagList {
    boolean d=false; // debugging switch for verbose output
    private Hashtable tagToRow;
    private String[] rowToTag;
    private int nr_of_tags;
    private Hashtable tagFrequency;
    private int sumTagFrequency;
    
    
    /** Creates a new instance of TagList */
    public TagList(String filename) {
       tagFrequency=new Hashtable();
       sumTagFrequency=0;
       try {
           load(filename);
      } catch (Exception e) {//System.err.println("File IO problem with "+filename+" :"+e.getMessage()); 
    	 
      }
        
        if (d) printTagList();
        
    } // end public TagList
    
       private void load(String filename) throws IOException, FileNotFoundException {
        int linecount = 0;
        String line;
        tagToRow=new Hashtable();
        
        
        BufferedReader lexFileReader1 = new BufferedReader(new FileReader(filename));
	
        // First pass: Count lines for array dimensioning
        
        
        while(lexFileReader1.readLine()!=null) {
		    linecount++;
        } // elihw
        lexFileReader1.close();
        nr_of_tags=linecount+1;
        
        // Initialize 
        rowToTag=new String[nr_of_tags];    
        tagToRow=new Hashtable();
        
        // and fill it
        BufferedReader lexFileReader2 = new BufferedReader(new FileReader(filename));
        
        linecount=0;
        while((line=lexFileReader2.readLine())!=null) {
            String[] items=line.split("\t");
            String newtag=items[0];
            if (items.length>1) {
                tagFrequency.put(newtag,new Integer(items[1]));
                sumTagFrequency+=(new Integer(items[1])).intValue();
            } else {
                tagFrequency.put(newtag,new Integer(1));              
                sumTagFrequency+=1;                
            }
            tagToRow.put(newtag, new Integer(linecount));
            rowToTag[linecount]=newtag;
            linecount++;
        } // elihw
        lexFileReader2.close();
        
          
        // invented dummy tag
        tagToRow.put("-", new Integer(linecount));
        rowToTag[linecount]="-";
        
    } // end private void load
    
    public int getNrOfTags() {
        return nr_of_tags;
    }  // end public int getNrOfTags()
 
    public int getCodeForTag(String tag) {
        int retval;
        
        if (tagToRow.containsKey(tag)) {
            retval=((Integer)tagToRow.get(tag)).intValue();
        } else {
            retval=-1;
        } // esle fi-        
        return retval;
    }  // end public int getCodeForTag(String tag)
    
    public String getTagForCode(int code) {
        String retStr= new String();
        if (code<nr_of_tags) {
            retStr=rowToTag[code];
        } else {
            retStr="undefined";
        } // esle fi-
        return retStr;
    } // end  public String getTagForCode(int code)
       
    
    /* returns tag probability without context */
    public double getTagProb(String tag) {
         
        double ret;
        if (tagToRow.containsKey(tag)) {
            ret=(((Integer)tagFrequency.get(tag)).doubleValue()+1.0)/(double)sumTagFrequency;
        } else {
            ret=0;
        } // esle fi-
        return ret;   
        
    } // end get Tasg prob
    
    
    public void printTagList() {
        System.out.println("---The "+nr_of_tags+" tags and their internal code ---");
        for (int r=0;r<nr_of_tags;r++) {
            System.out.println(rowToTag[r]+"\t"+r);
       
        } // rod// roff
       
       
    } // end printmatrix

    
    
} // end public class TagList
