/*
 * Transitions_fast.java
 *
 * Created on 1. Dezember 2005, 20:38
 *
 *
 *   * Format is 
     *  T1|T2 p(t1) p(t2) ... p(tn)
     *  T3|T4  ........
     *
     *  that means: after sequence T1 T2, the probability for tag with code 0 is p(t1).
     *
     *  All non-initialized entries are zeroed.
     *
 *
 */

package de.tudarmstadt.langtech.viterbitagger;
import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.FileReader;
/**
 *
 * @author BIEMANN
 */
public class Transitions_fast {
    boolean d=!true; // debugging
    double[][][] transmatrix;
    
    double SMOOTHVAL;  // dynamically retrermined smoothing value during loading. 

    TagList taglist; // the tag list
    int dimension; // number of tags
  
    /** Creates a new instance of Transistions */
    public Transitions_fast(String filename, TagList tl) {
        this.taglist=tl;
        this.dimension=tl.getNrOfTags();
        SMOOTHVAL=1;
        
        // The transition matrix is a 3-dimensional matrix, dimension is tagset size.
        transmatrix=new double[dimension][dimension][dimension];
        
        // init with 0s
        if (d) System.out.print("Initializing transition matrix with "+dimension+"^3="+dimension*dimension*dimension+" zeros..");
        for(int i=0;i<dimension;i++) {
         for(int j=0;j<dimension;j++) {             
          for(int k=0;k<dimension;k++) {
              transmatrix[i][j][k]=0; 
          } // rof k
         } // rof j
        } // rof i
        
        if (d) System.out.println("..done initializing.");
        
        try {
           load(filename);
        } catch (Exception e) {//System.err.println("File IO problem with "+filename+" :"+e.getMessage()); 
        	
        }
 
    } // end public Transitions(String filename, TagList tl) {
    
    /** loads transition file */
    private void load(String filename) throws IOException, FileNotFoundException {
                
        String line;
        String[] linearray;
        String[] tags;
        int tag1code, tag2code;
        
        BufferedReader FileReader = new BufferedReader(new FileReader(filename));

        
        while((line=FileReader.readLine())!=null) {
            linearray=line.split("\t");
            tags=linearray[0].split("\\|");
            tag1code=taglist.getCodeForTag(tags[0]);
            tag2code=taglist.getCodeForTag(tags[1]);
            
            if(d) System.out.println("prefix "+linearray[0]+" split into "+tags[0]+" and "+tags[1]+ ", codes "+tag1code+" and "+tag2code);
                
            for (int p=0;p<linearray.length-1;p++) {
               Double actPos= new Double(linearray[p+1]);
               double doubleActPos=actPos.doubleValue();
               transmatrix[tag1code][tag2code][p]=doubleActPos;
               
               if ((doubleActPos > 0.00000000000)&&(doubleActPos<SMOOTHVAL)) {
                   SMOOTHVAL=doubleActPos;
               }
            } // rof

        } // elihw
        
        // smoothing value = sampling error. This is +0.5 smoothing.
        SMOOTHVAL=SMOOTHVAL/2;
     
        if (d) System.out.println("Smoothing value: "+SMOOTHVAL);
        
        FileReader.close();
        
    } // end private void load
    
    
    /** this is main functionality: get a transition probability */ 
    public double getTransProb(int x, int y, int z) {
        double retVal;
        if((x<dimension)&&(y<dimension)&&(z<dimension)) {
            retVal=transmatrix[x][y][z]+SMOOTHVAL;
        } else {
           retVal=-1; // error
        } // esle fi.        
        return retVal;
    } // end public double getTransProb
       
       
} // end public class Transitions
